import java.util.*;
import java.io.*;

/** Slovnik, automaticke ohackovani vyrazu */
public class Vocabulary {
 /** defaultni encoding slovniku - pouzije se, pokud neni specifikovan jiny */
 private static final String DEFAULT_ENCODING="iso-8859-2";

 /** HashMap drzici slovnik */
 private HashMap voc=null;

 /** prekladova tabulka - znak z tabulky char_from odpovida neohackovanemu znaku v char_to */
 private static final String char_from="?̊؎?";
 /** prekladova tabulka - znak z tabulky char_from odpovida neohackovanemu znaku v char_to */
 private static final String char_to=  "escrzyaieuudtnoESCRZYAIEUUDTNO";

 /** vrati slovo, ve kterem je odstranena diakritika - hacky a carky */
 public String strip_diac(String word) {
  int i,ind,wl;
  char c;
  StringBuffer res=new StringBuffer(word);
  wl=word.length();
  for (i=0;i<wl;i++) {
   ind=char_from.indexOf(res.charAt(i));
   if (ind>=0) res.setCharAt(i,char_to.charAt(ind));
  }
  return res.toString();
 }

 /** nacte slovnik ze souboru file. Slovnik ma kodovani encoding. */
 private void init(File file,String encoding) throws Exception {
  if (encoding==null || encoding.equals("")) encoding=DEFAULT_ENCODING;
  voc=new HashMap();   
  BufferedReader f=new BufferedReader(new InputStreamReader(new FileInputStream(file),encoding));
  String line;
  String trans;
  while ((line = f.readLine()) != null) {
   trans=strip_diac(line);
   String orig=(String)voc.get(trans);
   if (orig==null) { //neni tam
    voc.put(trans,line);
   } else { //nejednoznacnost typu laska -> l'aska/lask'a
    int i;
    StringBuffer mod=new StringBuffer(line);//modifikuju abych ohackoval co jde a bylo to jednoznacne
    for (i=0;i<orig.length();i++) {
     //lisi se -> dosadim nehackovany znak     
     if (orig.charAt(i)!=mod.charAt(i)) mod.setCharAt(i,trans.charAt(i));
    }
    voc.put(trans,mod.toString());//zamenim
   }
  }
  f.close();
 }

 /** vypise na STDOUT zbytecna slova (tj. jejich preklad je stejny jako samo slovo)
     Hodi se na test kvality slovniku (v nem by takova slova byt nemela) */
 public void printUselessWords() {
  Iterator i=voc.keySet().iterator();
  while (i.hasNext()) {
   String s=(String)i.next();
   if (s.equals((String)voc.get(s))) System.out.println(s);
  }
 }

 /** konstruktor - novy slovnik se nahraje ze souboru filename */
 public Vocabulary(String filename) throws Exception  {
  File f=new File(filename);
  init(f,DEFAULT_ENCODING);
 }

 /** konstruktor - novy slovnik se nahraje ze souboru f */
 public Vocabulary(File f) throws Exception  {
  init(f,DEFAULT_ENCODING);
 }

 /** konstruktor - novy slovnik se nahraje ze souboru filename. Soubor ma kodovani encoding */
 public Vocabulary(String filename,String encoding) throws Exception  {
  File f=new File(filename);
  init(f,encoding);
 }

 /** konstruktor - novy slovnik se nahraje ze souboru f. Soubor ma kodovani encoding */
 public Vocabulary(File f,String encoding) throws Exception  {
  init(f,encoding);
 }

 /** Ohackuje dane slovo */
 public String processWord(String s) {
  if (s.equals("")) return "";
  String wo=(String)voc.get(s);
  if (wo!=null) return wo;
  return s;
 }

 /** Doplni diakritiku v dane vete - pro vsechny slova se pokusi najit jejich ekvivalent s diakritikou.
     Slova ktere nezna, nebo je nelze ohackovat jednoznacne (napr laska, zprava ...) se ohackuji pouze
     zcasti (pokud lze aspon cast ohackovat jednoznacne) nebo vubec */
 public String processSentence(String s) {
  int i;
  char c;
  StringBuffer res=new StringBuffer();
  StringBuffer word=new StringBuffer();
  for (i=0;i<s.length();i++) {
   c=s.charAt(i);
   if (Character.isLetter(c)) word.append(c); //pridat znak do slova
   else {
    res.append(processWord(word.toString()));
    word.setLength(0);
    res.append(c);
   }
  }
  res.append(processWord(word.toString()));
  return res.toString();
 }
}